<?php

namespace Drupal\file_download_token\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\ProxyClass\File\MimeType\MimeTypeGuesser;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file_download_token\DownloadTokenManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mime\Header\UnstructuredHeader;

/**
 * The token download controller.
 */
class TokenDownloadController implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The download token manager.
   *
   * @var \Drupal\file_download_token\DownloadTokenManagerInterface
   */
  protected DownloadTokenManagerInterface $downloadTokenManager;

  /**
   * The MIME type guesser.
   *
   * @var \Drupal\Core\ProxyClass\File\MimeType\MimeTypeGuesser
   */
  protected MimeTypeGuesser $mimeTypeGuesser;

  /**
   * Constructs a TokenDownloadController object.
   *
   * @param \Drupal\file_download_token\DownloadTokenManagerInterface $download_token_manager
   *   The download token manager.
   * @param \Drupal\Core\ProxyClass\File\MimeType\MimeTypeGuesser $mime_type_guesser
   *   The MIME type guesser.
   */
  public function __construct(DownloadTokenManagerInterface $download_token_manager, MimeTypeGuesser $mime_type_guesser) {
    $this->downloadTokenManager = $download_token_manager;
    $this->mimeTypeGuesser = $mime_type_guesser;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('file_download_token.download_token_manager'),
      $container->get('file.mime_type.guesser')
    );
  }

  /**
   * Grant download access to a file.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param string|null $token
   *   The given token to be checked.
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse|\Symfony\Component\HttpFoundation\Response
   *   The binary file response, or 404 http response.
   */
  public function download(RouteMatchInterface $route_match, ?string $token = NULL): BinaryFileResponse|Response {
    $file = $this->downloadTokenManager->getFile($token);
    if (!$file) {
      return new Response($this->t('File not found.'), 404);
    }

    $uri = $file->getFileUri();

    $mime = $this->mimeTypeGuesser->guessMimeType($uri);

    $name = new UnstructuredHeader('name', basename($uri));
    $filename = new UnstructuredHeader('filename', $file->label());

    $headers = [
      'Content-Type' => $mime . '; name="' . $name->getBodyAsString() . '"',
      'Content-Length' => filesize($uri),
      'Content-Disposition' => 'attachment; filename="' . $filename->getBodyAsString() . '"',
      'Cache-Control' => 'private',
    ];

    return new BinaryFileResponse($uri, 200, $headers, TRUE);

  }

}
