<?php

namespace Drupal\file_download_token;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\file\FileInterface;

/**
 * The download token manager.
 */
class DownloadTokenManager implements DownloadTokenManagerInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The time object.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * DownloadTokenManager constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $connection, LoggerChannelInterface $logger, TimeInterface $time) {
    $this->entityTypeManager = $entity_type_manager;
    $this->connection = $connection;
    $this->logger = $logger;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public function createToken(FileInterface $file) {
    $token = $this->generateToken();

    try {
      $this->connection
        ->insert('download_tokens')
        ->fields([
          'fid' => $file->id(),
          'token' => $token,
          'timestamp' => $this->time->getCurrentTime(),
        ])
        ->execute();
    }
    catch (\Exception $e) {
      $this->logger
        ->error('Error while generating token: @error', ['@error' => $e->getMessage()]);
    }

    return $token;
  }

  /**
   * {@inheritdoc}
   */
  public function createTokenUrl(FileInterface $file) {
    $token = $this->createToken($file);
    return Url::fromRoute(
      'file_download_token.download_file',
      ['token' => $token],
      ['absolute' => TRUE],
    );
  }

  /**
   * {@inheritdoc}
   */
  public function createTokenLink(FileInterface $file) {
    return Link::fromTextAndUrl(
      $this->t('Download @filename',
      ['@filename' => $file->label()]),
      $this->createTokenUrl($file),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function generateToken() {
    return Crypt::randomBytesBase64(55);
  }

  /**
   * {@inheritdoc}
   */
  public function getFile(string $token) {
    $result = $this->connection->select('download_tokens', 'dt')
      ->fields('dt')
      ->condition('token', $token)
      ->execute();

    if ($record = $result->fetch()) {
      return $this->entityTypeManager
        ->getStorage('file')
        ->load($record->fid);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function cleanupTokens() {
    $timestamp = $this->time->getCurrentTime();
    // We delete all tokens older than 24 hours.
    $timestamp -= 24 * 60 * 60;
    $this->deleteTokensOlderThan($timestamp);
  }

  /**
   * Deletes tokens that are older than the given timestamp.
   *
   * @param int $timestamp
   *   The given timestamp.
   */
  public function deleteTokensOlderThan($timestamp) {
    if ($timestamp === NULL) {
      $timestamp = $this->time->getCurrentTime();
    }

    $results = $this->connection->delete('download_tokens')
      ->condition('timestamp', $timestamp, '<=')
      ->execute();
  }

}
